Package: pubmedR
Title: Gathering Metadata About Publications, Grants, Clinical Trials
        from 'PubMed' Database
Version: 1.0.2
Authors@R: 
    person(given = "Massimo",
           family = "Aria",
           role = c("aut", "cre"),
           email = "massimo.aria@gmail.com",
           comment = c(ORCID = "0000-0002-8517-9411"))
Description: A set of tools to extract bibliographic content from 'PubMed' database
    using 'NCBI' REST API <https://www.ncbi.nlm.nih.gov/home/develop/api/>.
    It includes functions to search, download, and convert 'PubMed' bibliographic
    records into data frames compatible with the 'bibliometrix' package.
    Features include programmatic query building, batch downloading by PMID,
    citation enrichment via 'NCBI' E-Link, and robust error handling with
    automatic retry logic.
License: GPL-3
URL: https://github.com/massimoaria/pubmedR
BugReports: https://github.com/massimoaria/pubmedR/issues
Encoding: UTF-8
Imports: rentrez, XML
Suggests: bibliometrix, knitr, rmarkdown, testthat (>= 3.0.0), withr
Config/testthat/edition: 3
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-04 18:14:49 UTC; massimoaria
Author: Massimo Aria [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8517-9411>)
Maintainer: Massimo Aria <massimo.aria@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 21:10:32 UTC
Built: R 4.6.0; ; 2026-05-04 22:08:34 UTC; unix

# Generated by roxygen2: do not edit by hand

export(pmApi2df)
export(pmApiRequest)
export(pmCitedBy)
export(pmCollect)
export(pmEnrichCitations)
export(pmExtractReferences)
export(pmFetchById)
export(pmQueryBuild)
export(pmQueryTotalCount)
export(pmReferences)
import(XML)
import(rentrez)

#' Collect and process PubMed bibliographic data in one step
#'
#' A convenience wrapper that executes the full pubmedR workflow:
#' query building, record count check, metadata download, conversion to
#' data frame, and (optionally) citation enrichment via NCBI E-Link.
#'
#' @param query is a character. A PubMed search query in Entrez syntax.
#' Alternatively, if \code{terms} is provided, the query is built automatically
#' using \code{\link{pmQueryBuild}} and this argument is ignored.
#' @param terms is a character or character vector or NULL. Search terms passed to
#' \code{\link{pmQueryBuild}}. When provided, a query is built automatically
#' and the \code{query} argument is ignored. Default is \code{NULL}.
#' @param fields is a character or character vector. PubMed search tags used
#' when building the query from \code{terms}. Default is \code{"Title/Abstract"}.
#' @param language is a character or NULL. Language filter for query building.
#' Default is \code{NULL}.
#' @param pub_type is a character or NULL. Publication type filter for query building.
#' Default is \code{NULL}.
#' @param date_range is a character vector of length 2 or NULL. Date range
#' in format \code{c("YYYY", "YYYY")}. Default is \code{NULL}.
#' @param mesh_terms is a character or character vector or NULL. MeSH terms
#' for query building. Default is \code{NULL}.
#' @param limit is numeric. Maximum number of records to download.
#' Default is \code{2000}.
#' @param enrich is logical. If \code{TRUE}, citation counts (TC) and cited
#' references (CR) are added via \code{\link{pmEnrichCitations}}.
#' Default is \code{FALSE} because enrichment makes 2 API calls per article
#' and can be slow for large collections.
#' @param format is a character. Output format passed to \code{\link{pmApi2df}}.
#' Either \code{"bibliometrix"} (default) or \code{"raw"}.
#' @param api_key is a character or NULL. NCBI API key. Can also be set via
#' the environment variable \code{PUBMED_API_KEY} or \code{ENTREZ_KEY}.
#' Default is \code{NULL}.
#' @param batch_size is numeric. Records per API request. Default is 200.
#' @param verbose is logical. If \code{TRUE} (default), prints progress messages.
#'
#' @return a data frame containing bibliographic records, compatible with the
#' \code{bibliometrix} package when \code{format = "bibliometrix"}.
#'
#' @details
#' This function chains together the core pubmedR functions in the recommended order:
#' \enumerate{
#'   \item \strong{Query}: If \code{terms} is provided, builds the query with
#'         \code{\link{pmQueryBuild}}; otherwise uses the \code{query} string directly.
#'   \item \strong{Count}: Checks the total number of matching records with
#'         \code{\link{pmQueryTotalCount}}.
#'   \item \strong{Download}: Fetches metadata with \code{\link{pmApiRequest}}.
#'   \item \strong{Convert}: Transforms XML to a data frame with \code{\link{pmApi2df}}.
#'   \item \strong{Enrich} (optional): Adds citation data with
#'         \code{\link{pmEnrichCitations}}.
#' }
#'
#' @examples
#'
#' \donttest{
#' # Using a raw query string
#' M <- pmCollect(
#'   query = "bibliometric*[Title/Abstract] AND english[LA] AND 2020:2024[DP]",
#'   limit = 50
#' )
#'
#' # Using the query builder parameters
#' M <- pmCollect(
#'   terms = "bibliometric*",
#'   language = "english",
#'   pub_type = "Journal Article",
#'   date_range = c("2020", "2024"),
#'   limit = 50
#' )
#'
#' # With citation enrichment (slower, requires extra API calls)
#' M <- pmCollect(
#'   terms = "bibliometric*",
#'   date_range = c("2023", "2024"),
#'   limit = 10,
#'   enrich = TRUE
#' )
#' }
#'
#' @seealso \code{\link{pmQueryBuild}}, \code{\link{pmQueryTotalCount}},
#' \code{\link{pmApiRequest}}, \code{\link{pmApi2df}}, \code{\link{pmEnrichCitations}}
#'
#' @export

pmCollect <- function(query = NULL,
                      terms = NULL,
                      fields = "Title/Abstract",
                      language = NULL,
                      pub_type = NULL,
                      date_range = NULL,
                      mesh_terms = NULL,
                      limit = 2000,
                      enrich = FALSE,
                      format = "bibliometrix",
                      api_key = NULL,
                      batch_size = 200,
                      verbose = TRUE) {

  api_key <- get_api_key(api_key)

  ## Step 1: Build or validate query
  if (!is.null(terms)) {
    query <- pmQueryBuild(
      terms = terms,
      fields = fields,
      language = language,
      pub_type = pub_type,
      date_range = date_range,
      mesh_terms = mesh_terms
    )
    if (verbose) cat("Query: ", query, "\n\n")
  }

  if (is.null(query) || !nzchar(trimws(query))) {
    stop("A query string or search terms must be provided.", call. = FALSE)
  }

  ## Step 2: Count matching records
  res <- pmQueryTotalCount(query = query, api_key = api_key)

  if (verbose) {
    cat("Total records found:", res$total_count, "\n")
    cat("Records to download:", min(res$total_count, limit), "\n\n")
  }

  if (res$total_count == 0) {
    message("No records found. Returning empty data frame.")
    return(data.frame())
  }

  ## Step 3: Download metadata
  D <- pmApiRequest(
    query = query,
    limit = limit,
    api_key = api_key,
    batch_size = batch_size
  )

  ## Step 4: Convert to data frame
  M <- pmApi2df(D, format = format)

  if (verbose) {
    cat("\nRecords converted:", nrow(M), "\n")
  }

  ## Step 5: Enrich with citation data (optional)
  if (enrich) {
    if (verbose) cat("\nEnriching with citation data...\n")
    M <- pmEnrichCitations(M, api_key = api_key)
  }

  if (verbose) cat("\nDone.\n")

  return(M)
}

#' Gather bibliographic content from PubMed database using NCBI entrez APIs
#'
#' It gathers metadata about publications from the NCBI PubMed database.
#' The use of NCBI PubMed APIs is entirely free, and doesn't necessarily require an API key.
#' The function \code{pmApiRequest} queries NCBI PubMed using an entrez query formulated through
#' the Entrez query language or the helper function \code{\link{pmQueryBuild}}.
#'
#' Official API documentation is \href{https://www.ncbi.nlm.nih.gov/books/NBK25500/}{https://www.ncbi.nlm.nih.gov/books/NBK25500/}.
#' @param query is a character. It contains a search query formulated using the Entrez query language.
#' @param limit is numeric. It indicates the max number of records to download.
#' @param api_key is a character. It contains a valid API key for the NCBI E-utilities.
#' Default is \code{api_key=NULL}. The API key can also be set via the environment variable
#' \code{PUBMED_API_KEY} or \code{ENTREZ_KEY}.
#' @param batch_size is numeric. The number of records to download per API request. Default is 200.
#'
#' @return a list D composed by 5 objects:
#'\tabular{lll}{
#' data \tab \tab It is the xml-structured list containing the bibliographic metadata collection downloaded from the PubMed database.\cr
#' query \tab \tab It a character object containing the original query formulated by the user.\cr
#' query_translation \tab \tab It a character object containing the query, translated by the NCBI Automatic Terms Translation system and submitted to the PubMed database.\cr
#' records_downloaded \tab \tab It is an integer object indicating the total number of records downloaded and stored in "data".\cr
#' total_count \tab \tab It is an integer object indicating the total number of records matching the query (stored in the "query_translation" object").}
#'
#' To obtain a free access to NCBI API, please visit: \href{https://pmc.ncbi.nlm.nih.gov/tools/developers/}{https://pmc.ncbi.nlm.nih.gov/tools/developers/}
#'
#' To obtain more information about how to write a NCBI search query, please visit: \href{https://pubmed.ncbi.nlm.nih.gov/help/#search-tags}{https://pubmed.ncbi.nlm.nih.gov/help/#search-tags}
#'
#' @examples
#'
#' \donttest{
#' query <- "bibliometric*[Title/Abstract] AND english[LA]
#'           AND Journal Article[PT] AND 2000:2020[DP]"
#'  D <- pmApiRequest(query = query, limit = 100, api_key = NULL)
#' }
#'
#' @seealso \code{\link{pmQueryTotalCount}}
#' @seealso \code{\link{pmApi2df}}
#' @seealso \code{\link{pmQueryBuild}}
#'
#' @export
#' @import XML
#' @import rentrez

pmApiRequest <- function(query, limit, api_key = NULL, batch_size = 200) {
  api_key <- get_api_key(api_key)

  ## query total count
  res <- pmQueryTotalCount(query = query, api_key = api_key)

  if (res$total_count == 0) {
    message("No records found for the given query.")
    P <- list(
      data = list(),
      query = query,
      query_translation = res$query_translation,
      records_downloaded = 0,
      total_count = 0
    )
    return(P)
  }

  n <- min(res$total_count, limit)
  step <- min(n, batch_size)
  metadata <- list()
  s <- 0
  last_time <- NULL
  n_batches <- ceiling(n / step)

  cat("\nDownloading", n, "documents from PubMed...\n")
  pb <- utils::txtProgressBar(
    min = 0,
    max = n,
    initial = 0,
    char = "=",
    style = 3
  )

  ## download metadata in batches
  while (s < n) {
    current_step <- min(step, n - s)

    ## Rate limiting
    last_time <- api_throttle(api_key, last_time)

    ## Fetch with retry
    multi_summs <- api_call_with_retry(
      entrez_fetch(
        db = "pubmed",
        web_history = res$web_history,
        retstart = s,
        retmax = current_step,
        rettype = "xml",
        parsed = TRUE,
        api_key = api_key
      )
    )

    multi_summs <- xmlToList(multi_summs, simplify = FALSE)
    metadata <- c(metadata, multi_summs)

    s <- s + current_step
    utils::setTxtProgressBar(pb, s)
  }

  close(pb)
  cat("\nDownload completed.\n")

  P <- list(
    data = metadata,
    query = query,
    query_translation = res$query_translation,
    records_downloaded = n,
    total_count = res$total_count
  )
  return(P)
}

test_that("pmApi2df handles empty data", {
  P <- list(data = list())
  result <- pmApi2df(P)
  expect_true(is.data.frame(result))
  expect_equal(nrow(result), 0)
})

test_that("pmApi2df output has expected columns in bibliometrix format", {
  skip_on_cran()
  skip_if_offline()
  skip_if_not(
    nzchar(Sys.getenv("PUBMED_API_KEY")) || nzchar(Sys.getenv("ENTREZ_KEY")),
    "no PubMed API key set"
  )

  D <- pmApiRequest(
    query = "bibliometric*[Title/Abstract] AND english[LA] AND 2020:2020[DP]",
    limit = 5,
    api_key = NULL
  )
  M <- pmApi2df(D, format = "bibliometrix")

  expected_cols <- c("AU", "AF", "TI", "SO", "LA", "DT", "DE", "ID",
                     "AB", "C1", "CR", "TC", "SN", "J9", "JI", "PY",
                     "VL", "DI", "PG", "UT", "PMID", "DB", "AU_UN",
                     "AU_CO", "AU1_CO")
  for (col in expected_cols) {
    expect_true(col %in% names(M), info = paste("Missing column:", col))
  }
  expect_true(nrow(M) > 0)
  expect_equal(unique(M$DB), "PUBMED")
})

test_that("pmApi2df raw format does not add AU_CO columns", {
  skip_on_cran()
  skip_if_offline()
  skip_if_not(
    nzchar(Sys.getenv("PUBMED_API_KEY")) || nzchar(Sys.getenv("ENTREZ_KEY")),
    "no PubMed API key set"
  )

  D <- pmApiRequest(
    query = "bibliometric*[Title/Abstract] AND english[LA] AND 2020:2020[DP]",
    limit = 3,
    api_key = NULL
  )
  M <- pmApi2df(D, format = "raw")

  expect_false("AU_CO" %in% names(M))
  expect_false("AU1_CO" %in% names(M))
})

#' Convert xml PubMed bibliographic data into a dataframe
#'
#' It converts PubMed data, downloaded using Entrez API, into a dataframe
#'
#' @param P is a list following the xml PubMed structure, downloaded using the function \code{pmApiRequest}.
#' @param format is a character. If \code{format = "bibliometrix"} data will be converted in the bibliometrix compatible data format.
#' If \code{format = "raw"} data will save in a data frame without any other data editing procedure.
#'
#' @return a dataframe containing bibliographic records.
#'
#' To obtain a free access to NCBI API, please visit: \href{https://pmc.ncbi.nlm.nih.gov/tools/developers/}{https://pmc.ncbi.nlm.nih.gov/tools/developers/}
#'
#' To obtain more information about how to write a NCBI search query, please visit: \href{https://pubmed.ncbi.nlm.nih.gov/help/#search-tags}{https://pubmed.ncbi.nlm.nih.gov/help/#search-tags}
#'
#' @examples
#'
#' # Example: Querying a collection of publications
#'
#' \donttest{
#' query <- "bibliometric*[Title/Abstract] AND english[LA]
#'          AND Journal Article[PT] AND 2000:2020[DP]"
#' D <- pmApiRequest(query = query, limit = 100, api_key = NULL)
#' M <- pmApi2df(D)
#' }
#'
#' @seealso \code{\link{pmApiRequest}}
#' @seealso \code{\link{pmQueryTotalCount}}
#'
#' @export
pmApi2df <- function(P, format = "bibliometrix") {
  P <- P$data

  n <- length(P)

  if (n == 0) {
    message("No records to convert.")
    return(data.frame())
  }

  ### Data Conversion
  df <- data.frame(
    AU = rep(NA_character_, n),
    AF = NA_character_,
    TI = NA_character_,
    SO = NA_character_,
    SO_CO = NA_character_,
    LA = NA_character_,
    DT = NA_character_,
    DE = NA_character_,
    ID = NA_character_,
    MESH = NA_character_,
    AB = NA_character_,
    C1 = NA_character_,
    CR = NA_character_,
    TC = 0,
    SN = NA_character_,
    J9 = NA_character_,
    JI = NA_character_,
    PY = NA_integer_,
    PY_IS = NA_character_,
    VL = NA_character_,
    DI = NA_character_,
    PG = NA_character_,
    GRANT_ID = NA_character_,
    GRANT_ORG = NA_character_,
    UT = NA_character_,
    PMID = NA_character_,
    DB = "PUBMED",
    AU_UN = NA_character_,
    stringsAsFactors = FALSE
  )

  pb <- utils::txtProgressBar(
    min = 0,
    max = n,
    initial = 0,
    char = "=",
    style = 3
  )

  for (i in 1:n) {
    utils::setTxtProgressBar(pb, i)

    a <- list2char(P[[i]])
    items <- names(a)

    ## Language
    df$LA[i] <- a["MedlineCitation.Article.Language"]

    ## Document Type
    df$DT[i] <- a[
      "MedlineCitation.Article.PublicationTypeList.PublicationType.text"
    ]

    ## Title
    df$TI[i] <- a["MedlineCitation.Article.ArticleTitle"]

    ## Publication Year
    ind <- which(items == "PubmedData.History.PubMedPubDate.Year")
    if (length(ind) > 0) {
      df$PY[i] <- min(as.numeric(a[ind]), na.rm = TRUE)
    }
    df$PY_IS[i] <- a[
      "MedlineCitation.Article.Journal.JournalIssue.PubDate.Year"
    ]

    ## Authors
    AU_last_ind <- which(
      items == "MedlineCitation.Article.AuthorList.Author.LastName"
    )
    AU_first_ind <- which(
      items == "MedlineCitation.Article.AuthorList.Author.ForeName"
    )
    AU_init_ind <- which(
      items == "MedlineCitation.Article.AuthorList.Author.Initials"
    )

    if (length(AU_last_ind) > 0) {
      nameAF <- paste(a[AU_last_ind], a[AU_first_ind], sep = ", ")
      nameAU <- paste(a[AU_last_ind], a[AU_init_ind], sep = " ")
      df$AF[i] <- paste(nameAF, collapse = ";")
      df$AU[i] <- paste(nameAU, collapse = ";")
    }

    ## Affiliations - extract all author affiliations
    Aff_name_ind <- which(
      items ==
        "MedlineCitation.Article.AuthorList.Author.AffiliationInfo.Affiliation"
    )
    if (length(Aff_name_ind) > 0) {
      Affiliations <- a[Aff_name_ind]

      ## Remove email addresses from affiliations
      Affiliations <- vapply(
        Affiliations,
        function(l) {
          parts <- unlist(strsplit(l, ", "))
          paste(parts[!grepl("@", parts, fixed = TRUE)], collapse = ", ")
        },
        character(1),
        USE.NAMES = FALSE
      )

      ## Remove empty affiliations
      Affiliations <- Affiliations[nzchar(trimws(Affiliations))]

      ## Unique affiliations for C1
      unique_aff <- unique(Affiliations)
      df$C1[i] <- paste(unique_aff, collapse = ";")
      df$AU_UN[i] <- df$C1[i]
    }

    ## Keywords and MeSH
    DE_ind <- which(items == "MedlineCitation.KeywordList.Keyword.text")
    if (length(DE_ind) > 0) {
      df$DE[i] <- paste(a[DE_ind], collapse = ";")
    }

    ID_ind <- which(
      items == "MedlineCitation.MeshHeadingList.MeshHeading.DescriptorName.text"
    )
    if (length(ID_ind) > 0) {
      df$ID[i] <- df$MESH[i] <- paste(a[ID_ind], collapse = ";")
    }

    ## Abstract - handle structured abstracts
    ind <- which(items == "MedlineCitation.Article.Abstract.AbstractText.text")
    if (length(ind) > 0) {
      df$AB[i] <- paste(a[ind], collapse = " ")
    } else {
      ind <- which(items == "MedlineCitation.Article.Abstract.AbstractText")
      if (length(ind) > 0) {
        df$AB[i] <- paste(a[ind], collapse = " ")
      }
    }

    ## Journal
    df$SO[i] <- a["MedlineCitation.Article.Journal.Title"]
    df$JI[i] <- df$J9[i] <- a["MedlineCitation.Article.Journal.ISOAbbreviation"]
    df$SO_CO[i] <- a["MedlineCitation.MedlineJournalInfo.Country"]

    ## DOI
    doi_ind <- which(
      items == "PubmedData.ArticleIdList.ArticleId..attrs.IdType"
    )
    ind <- which(a[doi_ind] == "doi")
    if (length(ind) > 0) {
      doi_val_ind <- doi_ind[ind] - 1
      df$DI[i] <- a[doi_val_ind]
    }

    ## ISSN
    df$SN[i] <- a["MedlineCitation.Article.Journal.ISSN.text"]

    ## Pages
    df$PG[i] <- a["MedlineCitation.Article.Pagination.MedlinePgn"]

    ## Volume
    df$VL[i] <- a["MedlineCitation.Article.Journal.JournalIssue.Volume"]

    ## PMID
    df$UT[i] <- df$PMID[i] <- a["MedlineCitation.PMID.text"]

    ## Grants
    GR_ID <- which(items == "MedlineCitation.Article.GrantList.Grant.GrantID")
    if (length(GR_ID) > 0) {
      df$GRANT_ID[i] <- paste(a[GR_ID], collapse = ";")
    }
    GR_ORG <- which(items == "MedlineCitation.Article.GrantList.Grant.Agency")
    if (length(GR_ORG) > 0) {
      df$GRANT_ORG[i] <- paste(a[GR_ORG], collapse = ";")
    }
  }

  close(pb)

  if (format == "bibliometrix") {
    DI <- df$DI
    df <- data.frame(lapply(df, toupper), stringsAsFactors = FALSE)
    df$DI <- DI
    df$AU_CO <- NA_character_
    df$AU1_CO <- NA_character_
  }

  ### PY
  df$PY <- as.numeric(df$PY)

  ### TC
  df$TC <- as.numeric(df$TC)
  df$TC[is.na(df$TC)] <- 0

  ### Remove empty rows (no Document Type)
  df <- df[!is.na(df$DT), ]

  return(df)
}

#' Fetch PubMed records by PMID
#'
#' It downloads metadata for a set of PubMed articles identified by their PMID (PubMed Identifier).
#' This is useful for retrieving specific known articles, updating existing datasets,
#' or downloading records identified through other sources.
#'
#' @param pmids is a character or numeric vector. A vector of PubMed identifiers (PMIDs).
#' @param api_key is a character. It contains a valid API key for the NCBI E-utilities.
#' Default is \code{api_key=NULL}. The API key can also be set via the environment variable
#' \code{PUBMED_API_KEY} or \code{ENTREZ_KEY}.
#' @param batch_size is numeric. The number of records to download per API request. Default is 200.
#'
#' @return a list following the same structure as \code{\link{pmApiRequest}} output, containing:
#' \tabular{lll}{
#' data \tab \tab The xml-structured list containing the bibliographic metadata.\cr
#' query \tab \tab A character string describing the PMID-based query.\cr
#' query_translation \tab \tab Same as query for PMID-based searches.\cr
#' records_downloaded \tab \tab The total number of records downloaded.\cr
#' total_count \tab \tab The total number of PMIDs requested.}
#'
#' @details
#' The function uses the NCBI E-utilities \code{efetch} endpoint to retrieve records directly
#' by their PMIDs, without requiring a search query. Records are downloaded in batches
#' to respect API rate limits.
#'
#' The output is compatible with \code{\link{pmApi2df}} for conversion to a dataframe.
#'
#' @examples
#'
#' \donttest{
#' # Download specific articles by PMID
#' pmids <- c("34813985", "34813456", "34812345")
#' D <- pmFetchById(pmids = pmids)
#' M <- pmApi2df(D)
#' }
#'
#' @seealso \code{\link{pmApiRequest}}
#' @seealso \code{\link{pmApi2df}}
#'
#' @export
#' @import XML
#' @import rentrez

pmFetchById <- function(pmids, api_key = NULL, batch_size = 200) {

  api_key <- get_api_key(api_key)
  pmids <- as.character(unique(pmids))
  pmids <- pmids[nzchar(trimws(pmids))]

  if (length(pmids) == 0) {
    stop("No valid PMIDs provided.", call. = FALSE)
  }

  n <- length(pmids)
  metadata <- list()
  last_time <- NULL

  cat("\nDownloading", n, "documents from PubMed by PMID...\n")
  pb <- utils::txtProgressBar(min = 0, max = n, initial = 0, char = "=", style = 3)

  ## Download in batches
  s <- 1
  while (s <= n) {
    end <- min(s + batch_size - 1, n)
    batch_ids <- pmids[s:end]

    ## Rate limiting
    last_time <- api_throttle(api_key, last_time)

    ## Fetch with retry
    multi_summs <- api_call_with_retry(
      entrez_fetch(
        db = "pubmed",
        id = batch_ids,
        rettype = "xml",
        parsed = TRUE,
        api_key = api_key
      )
    )

    multi_summs <- xmlToList(multi_summs, simplify = FALSE)
    metadata <- c(metadata, multi_summs)

    utils::setTxtProgressBar(pb, end)
    s <- end + 1
  }

  close(pb)
  cat("\nDownload completed.\n")

  P <- list(
    data = metadata,
    query = paste("PMID:", paste(pmids[1:min(5, n)], collapse = ", "),
                  if (n > 5) paste0("... (", n, " total)") else ""),
    query_translation = paste(n, "PMIDs"),
    records_downloaded = length(metadata),
    total_count = n
  )
  return(P)
}

#' Extract references from PubMed XML records
#'
#' Walks a result returned by \code{\link{pmApiRequest}} or
#' \code{\link{pmFetchById}} and pulls the \code{<ReferenceList>} block out of
#' every record. Returns one row per cited \code{<Reference>}, carrying the
#' source PMID, the free-text citation, and (when present) the cited PMID and
#' DOI parsed from \code{<ArticleIdList>}.
#'
#' @param P A list following the PubMed XML structure produced by
#'   \code{pmApiRequest()} or \code{pmFetchById()}. Must contain a \code{$data}
#'   element with one entry per record.
#'
#' @return A data.frame with columns
#' \tabular{lll}{
#' source_pmid \tab \tab The PMID of the article that cites the reference.\cr
#' citation    \tab \tab The free-text \code{<Citation>} string from PubMed.\cr
#' pmid        \tab \tab The PMID of the cited reference (if available).\cr
#' doi         \tab \tab The DOI of the cited reference (if available).}
#'
#' Returns an empty data.frame (with the same schema) if no references are
#' found.
#'
#' @details
#' Reference data in PubMed XML is populated when the publisher submits a
#' \code{<ReferenceList>} block to NLM (which is now common, but not universal).
#' This function does not call any web API; it merely parses what is already
#' present in the XML. Use \code{\link{pmEnrichCitations}} to also resolve cited
#' PMIDs into structured WoS-style citation strings.
#'
#' @examples
#' \donttest{
#' D <- pmFetchById("37289732")
#' refs <- pmExtractReferences(D)
#' head(refs)
#' }
#'
#' @seealso \code{\link{pmEnrichCitations}}, \code{\link{pmFetchById}}
#'
#' @export
pmExtractReferences <- function(P) {
  records <- P$data
  if (is.null(records) || length(records) == 0) {
    return(data.frame(
      source_pmid = character(0),
      citation = character(0),
      pmid = character(0),
      doi = character(0),
      stringsAsFactors = FALSE
    ))
  }

  out <- list()
  for (i in seq_along(records)) {
    rec <- records[[i]]
    src_pmid <- extract_record_pmid(rec)
    refs <- extract_refs_from_record(rec)
    if (nrow(refs) == 0) next
    refs$source_pmid <- src_pmid
    out[[length(out) + 1]] <- refs[, c("source_pmid", "citation", "pmid", "doi")]
  }

  if (length(out) == 0) {
    return(data.frame(
      source_pmid = character(0),
      citation = character(0),
      pmid = character(0),
      doi = character(0),
      stringsAsFactors = FALSE
    ))
  }

  do.call(rbind, out)
}

#' @keywords internal
#' Flatten a nested list into a named character vector
#'
#' @param x A nested list (typically from XML parsing)
#' @param use.names Logical, whether to preserve names
#' @param classes Character vector of classes to include
#' @return A named character vector
#' @noRd
list2char <- function(x, use.names = TRUE, classes = "ANY") {
  lung <- sum(rapply(x, function(x) 1L, classes = classes))
  Ch <- vector("list", lung)
  i <- 0L
  items <- rapply(x, function(x) {
    i <<- i + 1L
    ## Sanitize encoding: convert to UTF-8, replacing invalid bytes
    if (is.character(x)) {
      x <- iconv(x, from = "", to = "UTF-8", sub = "")
    }
    Ch[[i]] <<- x
    TRUE
  }, classes = classes)
  if (use.names && !is.null(nm <- names(items)))
    names(Ch) <- nm
  Ch <- unlist(Ch)
  return(Ch)
}


#' Retrieve NCBI API key
#'
#' Looks for an API key in the following order:
#' 1. Explicit argument passed by user
#' 2. Environment variable PUBMED_API_KEY
#' 3. Environment variable ENTREZ_KEY (rentrez convention)
#'
#' @param api_key A character string or NULL
#' @return A character string (API key) or NULL
#' @noRd
get_api_key <- function(api_key = NULL) {
  if (!is.null(api_key) && nzchar(api_key)) {
    return(api_key)
  }
  key <- Sys.getenv("PUBMED_API_KEY", unset = "")
  if (nzchar(key)) return(key)
  key <- Sys.getenv("ENTREZ_KEY", unset = "")
  if (nzchar(key)) return(key)
  return(NULL)
}


#' Rate-limited pause between API requests
#'
#' Enforces NCBI rate limits: 3 requests/sec without API key,
#' 10 requests/sec with API key.
#'
#' @param api_key A character string or NULL
#' @param last_request_time POSIXct time of the last request
#' @return POSIXct time after the pause (to be used as next last_request_time)
#' @noRd
api_throttle <- function(api_key = NULL, last_request_time = NULL) {
  min_interval <- if (is.null(api_key)) 1 / 3 else 1 / 10
  if (!is.null(last_request_time)) {
    elapsed <- as.numeric(Sys.time() - last_request_time, units = "secs")
    if (elapsed < min_interval) {
      Sys.sleep(min_interval - elapsed)
    }
  }
  return(Sys.time())
}


#' Extract fields from a PubmedBookArticle flattened record
#'
#' Maps the \code{BookDocument.*} / \code{PubmedBookData.*} XML structure
#' (used by GeneReviews, StatPearls, etc.) onto the same data frame columns
#' produced for \code{PubmedArticle} records.
#'
#' @param a A named character vector from \code{list2char}.
#' @param items \code{names(a)}.
#' @return A named list of column values for a single row.
#' @noRd
extract_book_article <- function(a, items) {
  row <- list()

  ## Language
  row$LA <- a["BookDocument.Language"]

  ## Document Type
  dt <- a["BookDocument.PublicationType.text"]
  if (is.na(dt)) dt <- "BOOK CHAPTER"
  row$DT <- dt

  ## Title (chapter)
  ti <- a["BookDocument.ArticleTitle.text"]
  if (is.na(ti)) ti <- a["BookDocument.ArticleTitle"]
  row$TI <- ti

  ## Publication Year: History first, then ContributionDate, then Book.PubDate
  ind <- which(items == "PubmedBookData.History.PubMedPubDate.Year")
  if (length(ind) > 0) {
    row$PY <- suppressWarnings(min(as.numeric(a[ind]), na.rm = TRUE))
  } else {
    py <- a["BookDocument.ContributionDate.Year"]
    if (is.na(py)) py <- a["BookDocument.Book.PubDate.Year"]
    if (!is.na(py)) row$PY <- as.numeric(py)
  }
  row$PY_IS <- a["BookDocument.Book.PubDate.Year"]

  ## Authors of the chapter (NOT the Book's editors under Book.AuthorList)
  AU_last_ind <- which(items == "BookDocument.AuthorList.Author.LastName")
  AU_first_ind <- which(items == "BookDocument.AuthorList.Author.ForeName")
  AU_init_ind <- which(items == "BookDocument.AuthorList.Author.Initials")
  if (length(AU_last_ind) > 0) {
    nameAF <- paste(a[AU_last_ind], a[AU_first_ind], sep = ", ")
    nameAU <- paste(a[AU_last_ind], a[AU_init_ind], sep = " ")
    row$AF <- paste(nameAF, collapse = ";")
    row$AU <- paste(nameAU, collapse = ";")
  }

  ## Affiliations
  Aff_name_ind <- which(
    items == "BookDocument.AuthorList.Author.AffiliationInfo.Affiliation"
  )
  if (length(Aff_name_ind) > 0) {
    Affiliations <- a[Aff_name_ind]
    Affiliations <- vapply(
      Affiliations,
      function(l) {
        parts <- unlist(strsplit(l, ", "))
        paste(parts[!grepl("@", parts, fixed = TRUE)], collapse = ", ")
      },
      character(1),
      USE.NAMES = FALSE
    )
    Affiliations <- Affiliations[nzchar(trimws(Affiliations))]
    unique_aff <- unique(Affiliations)
    row$C1 <- paste(unique_aff, collapse = ";")
    row$AU_UN <- row$C1
  }

  ## Keywords
  DE_ind <- which(items == "BookDocument.KeywordList.Keyword.text")
  if (length(DE_ind) > 0) {
    row$DE <- paste(a[DE_ind], collapse = ";")
  }

  ## Abstract
  ind <- which(items == "BookDocument.Abstract.AbstractText.text")
  if (length(ind) > 0) {
    row$AB <- paste(a[ind], collapse = " ")
  } else {
    ind <- which(items == "BookDocument.Abstract.AbstractText")
    if (length(ind) > 0) {
      row$AB <- paste(a[ind], collapse = " ")
    }
  }

  ## Source = Book title
  book_title <- a["BookDocument.Book.BookTitle.text"]
  if (is.na(book_title)) book_title <- a["BookDocument.Book.BookTitle"]
  row$SO <- book_title
  row$JI <- book_title
  row$J9 <- book_title

  ## Country / publisher location
  row$SO_CO <- a["BookDocument.Book.Publisher.PublisherLocation"]

  ## DOI (usually absent on book chapters but possible)
  doi_ind <- which(
    items == "PubmedBookData.ArticleIdList.ArticleId..attrs.IdType"
  )
  ind <- which(a[doi_ind] == "doi")
  if (length(ind) > 0) {
    doi_val_ind <- doi_ind[ind] - 1
    row$DI <- a[doi_val_ind]
  }

  ## PMID
  row$UT <- row$PMID <- a["BookDocument.PMID.text"]

  ## Grants
  GR_ID <- which(items == "BookDocument.GrantList.Grant.GrantID")
  if (length(GR_ID) > 0) {
    row$GRANT_ID <- paste(a[GR_ID], collapse = ";")
  }
  GR_ORG <- which(items == "BookDocument.GrantList.Grant.Agency")
  if (length(GR_ORG) > 0) {
    row$GRANT_ORG <- paste(a[GR_ORG], collapse = ";")
  }

  row
}


#' Null-coalescing operator
#' @noRd
`%||%` <- function(a, b) if (is.null(a)) b else a


#' Extract the source PMID from a parsed XML record
#'
#' Handles both \code{PubmedArticle} and \code{PubmedBookArticle} structures.
#'
#' @param rec A single record from \code{pmApiRequest()$data} or
#'   \code{pmFetchById()$data}.
#' @return A character PMID, or \code{NA_character_} if not found.
#' @noRd
extract_record_pmid <- function(rec) {
  pm <- rec$MedlineCitation$PMID
  if (is.null(pm)) pm <- rec$BookDocument$PMID
  if (is.null(pm)) return(NA_character_)
  if (is.list(pm)) pm <- pm$text %||% pm[[1]]
  pm <- as.character(pm)
  if (length(pm) == 0 || !nzchar(pm)) NA_character_ else pm[1]
}


#' Extract references from a parsed XML record
#'
#' Walks \code{PubmedData$ReferenceList} (or \code{PubmedBookData$ReferenceList})
#' and returns one row per cited \code{<Reference>}. Each row carries the
#' free-text citation plus PMID / DOI parsed from any \code{<ArticleIdList>}.
#'
#' @param rec A single record from \code{pmApiRequest()$data} or
#'   \code{pmFetchById()$data}.
#' @return A data.frame with columns \code{citation}, \code{pmid}, \code{doi}.
#' @noRd
extract_refs_from_record <- function(rec) {
  refs <- rec$PubmedData$ReferenceList
  if (is.null(refs)) refs <- rec$PubmedBookData$ReferenceList
  if (is.null(refs) || length(refs) == 0) return(empty_refs_df())

  out <- vector("list", length(refs))
  k <- 0L
  for (j in seq_along(refs)) {
    if (!identical(names(refs)[j], "Reference")) next
    r <- refs[[j]]

    citation <- r$Citation
    if (is.list(citation)) citation <- citation$text %||% paste(unlist(citation), collapse = " ")
    citation <- if (is.null(citation)) NA_character_ else as.character(citation)

    pmid <- NA_character_
    doi <- NA_character_
    aid_list <- r$ArticleIdList
    if (!is.null(aid_list)) {
      for (m in seq_along(aid_list)) {
        if (!identical(names(aid_list)[m], "ArticleId")) next
        ai <- aid_list[[m]]
        if (is.list(ai)) {
          id_val  <- ai$text %||% ai[[1]]
          id_type <- if (!is.null(ai$.attrs)) ai$.attrs[["IdType"]] else NULL
        } else {
          id_val  <- as.character(ai)
          id_type <- attr(ai, "IdType")
        }
        if (is.null(id_type) || is.null(id_val)) next
        if (identical(id_type, "pubmed") && is.na(pmid)) pmid <- as.character(id_val)
        if (identical(id_type, "doi")    && is.na(doi))  doi  <- as.character(id_val)
      }
    }

    k <- k + 1L
    out[[k]] <- data.frame(
      citation = citation,
      pmid = pmid,
      doi = doi,
      stringsAsFactors = FALSE
    )
  }
  if (k == 0L) return(empty_refs_df())
  do.call(rbind, out[seq_len(k)])
}


#' Empty references data.frame schema
#' @noRd
empty_refs_df <- function() {
  data.frame(
    citation = character(0),
    pmid = character(0),
    doi = character(0),
    stringsAsFactors = FALSE
  )
}


#' Build a WoS-style CR string from a \code{pmApi2df} row
#'
#' Produces "AUTHOR YYYY, JOURNAL ABBREV, V##, P##, DOI ..." as understood by
#' downstream bibliometrix functions.
#'
#' @param row A list (or one-row data.frame coerced to list) with at least the
#'   AU/PY/J9/JI/VL/PG/DI columns produced by \code{pmApi2df}.
#' @return Character WoS-style citation, or \code{NA_character_} when nothing
#'   meaningful can be derived.
#' @noRd
build_wos_cr_string <- function(row) {
  parts <- character(0)

  AU <- row$AU
  if (!is.null(AU) && !is.na(AU) && nzchar(AU)) {
    first_au <- trimws(strsplit(AU, ";", fixed = TRUE)[[1]][1])
    if (length(first_au) && !is.na(first_au) && nzchar(first_au)) {
      parts <- c(parts, toupper(first_au))
    }
  }

  PY <- row$PY
  if (!is.null(PY) && !is.na(PY) && nzchar(as.character(PY))) {
    parts <- c(parts, as.character(PY))
  }

  src <- row$J9
  if (is.null(src) || is.na(src) || !nzchar(src)) src <- row$JI
  if (is.null(src) || is.na(src) || !nzchar(src)) src <- row$SO
  if (!is.null(src) && !is.na(src) && nzchar(src)) parts <- c(parts, toupper(src))

  VL <- row$VL
  if (!is.null(VL) && !is.na(VL) && nzchar(VL)) parts <- c(parts, paste0("V", VL))

  PG <- row$PG
  if (!is.null(PG) && !is.na(PG) && nzchar(PG)) {
    fp <- trimws(strsplit(PG, "-", fixed = TRUE)[[1]][1])
    if (length(fp) && !is.na(fp) && nzchar(fp)) parts <- c(parts, paste0("P", fp))
  }

  DI <- row$DI
  if (!is.null(DI) && !is.na(DI) && nzchar(DI)) {
    doi_clean <- gsub("^https?://doi\\.org/", "", DI)
    parts <- c(parts, paste0("DOI ", toupper(doi_clean)))
  }

  if (length(parts) == 0) return(NA_character_)
  paste(parts, collapse = ", ")
}


#' Execute an API call with retry logic
#'
#' Wraps an expression with error handling and exponential backoff retry.
#'
#' @param expr An expression to evaluate (typically an API call)
#' @param max_retries Maximum number of retry attempts
#' @param verbose Logical, whether to print retry messages
#' @return The result of the expression
#' @noRd
api_call_with_retry <- function(expr, max_retries = 3, verbose = TRUE) {
  for (attempt in seq_len(max_retries + 1)) {
    result <- tryCatch(
      expr,
      error = function(e) {
        if (attempt > max_retries) {
          stop(sprintf(
            "API request failed after %d attempts. Last error: %s",
            max_retries + 1, conditionMessage(e)
          ), call. = FALSE)
        }
        wait_time <- min(2^(attempt - 1), 30)
        if (verbose) {
          message(sprintf(
            "API request failed (attempt %d/%d): %s. Retrying in %ds...",
            attempt, max_retries + 1, conditionMessage(e), wait_time
          ))
        }
        Sys.sleep(wait_time)
        return(NULL)
      }
    )
    if (!is.null(result)) return(result)
  }
}

test_that("list2char flattens nested lists correctly", {
  x <- list(a = list(b = "hello", c = "world"), d = "foo")
  result <- pubmedR:::list2char(x)
  expect_type(result, "character")
  expect_true(length(result) == 3)
  expect_equal(unname(result[1]), "hello")
})

test_that("get_api_key returns explicit key first", {
  result <- pubmedR:::get_api_key("my_key_123")
  expect_equal(result, "my_key_123")
})

test_that("get_api_key returns NULL when no key available", {
  withr::with_envvar(
    c(PUBMED_API_KEY = "", ENTREZ_KEY = ""),
    {
      result <- pubmedR:::get_api_key(NULL)
      expect_null(result)
    }
  )
})

test_that("get_api_key reads PUBMED_API_KEY env var", {
  withr::with_envvar(
    c(PUBMED_API_KEY = "env_key_123", ENTREZ_KEY = ""),
    {
      result <- pubmedR:::get_api_key(NULL)
      expect_equal(result, "env_key_123")
    }
  )
})

test_that("get_api_key reads ENTREZ_KEY as fallback", {
  withr::with_envvar(
    c(PUBMED_API_KEY = "", ENTREZ_KEY = "entrez_key_456"),
    {
      result <- pubmedR:::get_api_key(NULL)
      expect_equal(result, "entrez_key_456")
    }
  )
})

test_that("api_throttle enforces rate limiting", {
  t1 <- Sys.time()
  t2 <- pubmedR:::api_throttle(api_key = NULL, last_request_time = t1)
  elapsed <- as.numeric(t2 - t1, units = "secs")
  # Without API key, should wait at least ~0.33s

  expect_true(elapsed >= 0.3)
})

test_that("api_throttle is faster with API key", {
  t1 <- Sys.time()
  t2 <- pubmedR:::api_throttle(api_key = "key", last_request_time = t1)
  elapsed <- as.numeric(t2 - t1, units = "secs")
  # With API key, should wait at least ~0.1s but less than 0.33s
  expect_true(elapsed >= 0.09)
  expect_true(elapsed < 0.33)
})

#' Count the number of documents returned by a query
#'
#' It counts the number of documents that a query returns from the NCBI PubMed database.
#'
#' @param query is a character. It contains a search query formulated using the Entrez query language.
#' @param api_key is a character. It contains a valid API key for the NCBI E-utilities. Default is \code{api_key=NULL}.
#' The use of NCBI PubMed APIs is entirely free, and doesn't necessarily require an API key.
#' The API key can also be set via the environment variable \code{PUBMED_API_KEY} or \code{ENTREZ_KEY}.
#'
#' @return a list. It contains three objects:
#' \tabular{lll}{
#' total_count \tab  \tab The total number of records returned by the query\cr
#' query_translation \tab       \tab The query translation by the NCBI Automatic Terms Translation system \cr
#' web_history \tab      \tab The web history object. The NCBI provides search history features, which is useful for dealing with large lists of IDs or repeated searches.}
#'
#' To obtain a free access to NCBI API, please visit: \href{https://pmc.ncbi.nlm.nih.gov/tools/developers/}{https://pmc.ncbi.nlm.nih.gov/tools/developers/}
#'
#'
#' @examples
#'
#' \donttest{
#' query <- "bibliometric*[Title/Abstract] AND english[LA]
#'            AND Journal Article[PT] AND 2000:2020[DP]"
#' D <- pmQueryTotalCount(query = query, api_key = NULL)
#' }
#'
#' @seealso \code{\link{pmApiRequest}}
#' @seealso \code{\link{pmApi2df}}
#'
#' @export
pmQueryTotalCount <- function(query, api_key = NULL) {
  api_key <- get_api_key(api_key)

  ## Validate query
  terms <- unlist(lapply(strsplit(query, "\\["), function(l) trimws(l[[1]])))
  if (all(terms %in% c("", " "))) {
    results <- list(
      total_count = 0,
      query_translation = "Not valid search terms",
      web_history = NA
    )
    return(results)
  }

  ## Search with retry logic
  d_search <- api_call_with_retry(
    entrez_search(
      db = "pubmed",
      term = query,
      retmax = 0,
      api_key = api_key,
      use_history = TRUE
    )
  )

  results <- list(
    total_count = d_search$count,
    query_translation = d_search$QueryTranslation,
    web_history = d_search$web_history
  )
  return(results)
}

test_that("pmQueryTotalCount rejects empty query", {
  result <- pmQueryTotalCount(query = "[Title/Abstract]", api_key = NULL)
  expect_equal(result$total_count, 0)
  expect_equal(result$query_translation, "Not valid search terms")
})

test_that("pmQueryTotalCount rejects blank query", {
  result <- pmQueryTotalCount(query = " [Title/Abstract]", api_key = NULL)
  expect_equal(result$total_count, 0)
})

# Integration tests - require network access
test_that("pmQueryTotalCount returns results for valid query", {
  skip_on_cran()
  skip_if_offline()
  skip_if_not(
    nzchar(Sys.getenv("PUBMED_API_KEY")) || nzchar(Sys.getenv("ENTREZ_KEY")),
    "no PubMed API key set"
  )

  result <- pmQueryTotalCount(
    query = "bibliometric*[Title/Abstract] AND english[LA] AND 2020:2020[DP]",
    api_key = NULL
  )
  expect_true(result$total_count > 0)
  expect_type(result$query_translation, "character")
  expect_false(is.null(result$web_history))
})

#' Find articles that cite a given PubMed article
#'
#' It retrieves the PMIDs of articles that cite a given PubMed article,
#' using the NCBI E-Link service (PubMed Cited by).
#'
#' @param pmid is a character or numeric. A single PubMed identifier (PMID).
#' @param api_key is a character. It contains a valid API key for the NCBI E-utilities.
#' Default is \code{api_key=NULL}. The API key can also be set via the environment variable
#' \code{PUBMED_API_KEY} or \code{ENTREZ_KEY}.
#'
#' @return a list containing:
#' \tabular{lll}{
#' pmid \tab \tab The queried PMID.\cr
#' cited_by \tab \tab A character vector of PMIDs that cite the queried article.\cr
#' count \tab \tab The number of citing articles found.}
#'
#' @details
#' This function uses the NCBI E-Link endpoint with linkname "pubmed_pubmed_citedin"
#' to find articles in PubMed that cite the given article.
#'
#' Note: Citation data in PubMed is based on PubMed Central (PMC) and may not be as
#' comprehensive as commercial citation databases (e.g. Web of Science, Scopus).
#'
#' @examples
#'
#' \donttest{
#' # Find articles that cite PMID 25824007
#' cites <- pmCitedBy(pmid = "25824007")
#' cites$count
#' cites$cited_by
#' }
#'
#' @seealso \code{\link{pmReferences}}
#' @seealso \code{\link{pmFetchById}}
#'
#' @export
#' @import rentrez

pmCitedBy <- function(pmid, api_key = NULL) {

  api_key <- get_api_key(api_key)
  pmid <- as.character(pmid)[1]

  links <- api_call_with_retry(
    entrez_link(
      dbfrom = "pubmed",
      id = pmid,
      db = "pubmed",
      linkname = "pubmed_pubmed_citedin",
      api_key = api_key
    )
  )

  cited_by <- character(0)
  if (length(links$links$pubmed_pubmed_citedin) > 0) {
    cited_by <- links$links$pubmed_pubmed_citedin
  }

  result <- list(
    pmid = pmid,
    cited_by = cited_by,
    count = length(cited_by)
  )
  return(result)
}


#' Find references cited by a given PubMed article
#'
#' It retrieves the PMIDs of articles that are cited by (referenced in) a given
#' PubMed article, using the NCBI E-Link service.
#'
#' @param pmid is a character or numeric. A single PubMed identifier (PMID).
#' @param api_key is a character. It contains a valid API key for the NCBI E-utilities.
#' Default is \code{api_key=NULL}. The API key can also be set via the environment variable
#' \code{PUBMED_API_KEY} or \code{ENTREZ_KEY}.
#'
#' @return a list containing:
#' \tabular{lll}{
#' pmid \tab \tab The queried PMID.\cr
#' references \tab \tab A character vector of PMIDs referenced by the queried article.\cr
#' count \tab \tab The number of references found.}
#'
#' @details
#' This function uses the NCBI E-Link endpoint with linkname "pubmed_pubmed_refs"
#' to find articles in PubMed that are referenced by the given article.
#'
#' Note: Reference data is extracted from PubMed Central (PMC) full-text articles and
#' is only available when the full text is deposited in PMC. Not all PubMed articles
#' have reference data available.
#'
#' @examples
#'
#' \donttest{
#' # Find references of PMID 25824007
#' refs <- pmReferences(pmid = "25824007")
#' refs$count
#' refs$references
#' }
#'
#' @seealso \code{\link{pmCitedBy}}
#' @seealso \code{\link{pmFetchById}}
#'
#' @export
#' @import rentrez

pmReferences <- function(pmid, api_key = NULL) {

  api_key <- get_api_key(api_key)
  pmid <- as.character(pmid)[1]

  links <- api_call_with_retry(
    entrez_link(
      dbfrom = "pubmed",
      id = pmid,
      db = "pubmed",
      linkname = "pubmed_pubmed_refs",
      api_key = api_key
    )
  )

  references <- character(0)
  if (length(links$links$pubmed_pubmed_refs) > 0) {
    references <- links$links$pubmed_pubmed_refs
  }

  result <- list(
    pmid = pmid,
    references = references,
    count = length(references)
  )
  return(result)
}


#' Enrich a PubMed dataframe with citation data
#'
#' Adds cited references (CR field), reference counts (NR field), and
#' optionally citation counts (TC field) to a dataframe created by
#' \code{\link{pmApi2df}}.
#'
#' @param df is a dataframe. A bibliometric dataframe produced by
#'   \code{\link{pmApi2df}}.
#' @param P is the optional list returned by \code{\link{pmApiRequest}} or
#'   \code{\link{pmFetchById}} that produced \code{df}. Reuse it to avoid an
#'   extra round-trip when references are already available. If \code{NULL},
#'   the function calls \code{pmFetchById} on \code{df$PMID} to retrieve the
#'   underlying XML.
#' @param api_key is a character. It contains a valid API key for the NCBI
#'   E-utilities. Default is \code{api_key=NULL}. The API key can also be set
#'   via the environment variable \code{PUBMED_API_KEY} or \code{ENTREZ_KEY}.
#' @param resolve_pmids logical. When \code{TRUE} (default) the function fetches
#'   metadata for every cited PMID present in the references and assembles
#'   structured WoS-style citation strings ("AUTHOR YYYY, JOURNAL, V##, P##,
#'   DOI ..."). When \code{FALSE} the free-text \code{<Citation>} block from
#'   the XML is used as-is.
#' @param only_multiple logical. When \code{TRUE} only references cited by more
#'   than one source article are resolved to metadata (faster and cheaper).
#'   References not resolved keep their free-text citation. Defaults to
#'   \code{FALSE}.
#' @param include_TC logical. When \code{TRUE} (default) also call
#'   \code{\link{pmCitedBy}} for each source article and write the citation
#'   count to \code{df$TC}. Disable to skip this step.
#' @param batch_size integer. Number of records per API call when fetching
#'   metadata. Defaults to 200 (NCBI's hard cap for efetch).
#'
#' @return The input dataframe with updated \code{CR} (cited references),
#'   \code{NR} (number of references), and \code{TC} (times cited, if
#'   \code{include_TC = TRUE}) fields.
#'
#' @details
#' Cited references are extracted from the article's PubMed XML
#' (\code{<ReferenceList>}). This is more reliable than the previous E-Link
#' \code{pubmed_pubmed_refs} approach, which only worked for articles
#' deposited in PMC. References whose XML carries an \code{ArticleId}
#' \code{IdType="pubmed"} are resolved to bibliographic metadata in
#' batched \code{efetch} requests so that \code{CR} matches the WoS
#' convention used by bibliometrix; references with only free-text citations
#' are kept verbatim (uppercased).
#'
#' @examples
#' \donttest{
#' query <- "bibliometric*[Title/Abstract] AND english[LA]
#'          AND Journal Article[PT] AND 2000:2020[DP]"
#' D <- pmApiRequest(query = query, limit = 10, api_key = NULL)
#' M <- pmApi2df(D)
#' M <- pmEnrichCitations(M, P = D)        # avoid the extra fetch
#' }
#'
#' @seealso \code{\link{pmExtractReferences}}, \code{\link{pmCitedBy}},
#'   \code{\link{pmFetchById}}, \code{\link{pmApi2df}}
#'
#' @export
pmEnrichCitations <- function(df,
                              P = NULL,
                              api_key = NULL,
                              resolve_pmids = TRUE,
                              only_multiple = FALSE,
                              include_TC = TRUE,
                              batch_size = 200) {

  api_key <- get_api_key(api_key)

  if (!"PMID" %in% names(df) || nrow(df) == 0) {
    message("No valid PMID field found in the dataframe.")
    return(df)
  }

  pmids <- as.character(df$PMID)
  valid_pmids <- pmids[!is.na(pmids) & nzchar(pmids)]

  ## ----- Step 1: fetch full XML if not provided ---------------------------
  if (is.null(P)) {
    if (length(valid_pmids) == 0) {
      message("No valid PMIDs to enrich.")
      return(df)
    }
    cat("\nFetching XML for", length(valid_pmids), "articles...\n")
    P <- pmFetchById(valid_pmids, api_key = api_key, batch_size = batch_size)
  }

  ## ----- Step 2: extract references per source PMID -----------------------
  records <- P$data
  cat("\nExtracting references from XML...\n")
  pb <- utils::txtProgressBar(min = 0, max = max(length(records), 1), style = 3)
  pmid_to_refs <- list()
  for (i in seq_along(records)) {
    rec <- records[[i]]
    src <- extract_record_pmid(rec)
    if (!is.na(src) && nzchar(src)) {
      pmid_to_refs[[src]] <- extract_refs_from_record(rec)
    }
    utils::setTxtProgressBar(pb, i)
  }
  close(pb)

  ## ----- Step 3: collect unique cited PMIDs and resolve metadata ----------
  cited_lookup <- character(0)
  if (isTRUE(resolve_pmids)) {
    all_cited <- unlist(lapply(pmid_to_refs, function(d) d$pmid), use.names = FALSE)
    all_cited <- all_cited[!is.na(all_cited) & nzchar(all_cited)]

    if (isTRUE(only_multiple)) {
      freq <- table(all_cited)
      unique_cited <- names(freq[freq > 1])
    } else {
      unique_cited <- unique(all_cited)
    }

    if (length(unique_cited) > 0) {
      cat("\nResolving metadata for", length(unique_cited), "unique cited PMIDs...\n")
      cited_xml <- tryCatch(
        pmFetchById(unique_cited, api_key = api_key, batch_size = batch_size),
        error = function(e) NULL
      )
      if (!is.null(cited_xml)) {
        cited_df <- tryCatch(
          pmApi2df(cited_xml, format = "bibliometrix"),
          error = function(e) NULL
        )
        if (!is.null(cited_df) && nrow(cited_df) > 0) {
          for (i in seq_len(nrow(cited_df))) {
            cr <- build_wos_cr_string(as.list(cited_df[i, , drop = FALSE]))
            key <- as.character(cited_df$PMID[i])
            if (!is.na(cr) && nzchar(key)) cited_lookup[key] <- cr
          }
        }
      }
    }
  }

  ## ----- Step 4: assemble CR / NR per row ---------------------------------
  if (!"CR" %in% names(df)) df$CR <- NA_character_
  if (!"NR" %in% names(df)) df$NR <- 0L

  for (i in seq_along(pmids)) {
    p <- pmids[i]
    if (is.na(p) || !nzchar(p)) next
    refs <- pmid_to_refs[[p]]
    if (is.null(refs) || nrow(refs) == 0) next

    cr_strs <- vapply(seq_len(nrow(refs)), function(j) {
      r <- refs[j, , drop = FALSE]
      cited_pmid <- as.character(r$pmid)
      if (!is.na(cited_pmid) && nzchar(cited_pmid) &&
          cited_pmid %in% names(cited_lookup)) {
        return(cited_lookup[[cited_pmid]])
      }
      ctxt <- r$citation
      if (!is.na(ctxt) && nzchar(ctxt)) return(toupper(trimws(ctxt)))
      NA_character_
    }, character(1))

    cr_strs <- cr_strs[!is.na(cr_strs) & nzchar(cr_strs)]
    if (length(cr_strs) == 0) next

    df$CR[i] <- paste(cr_strs, collapse = "; ")
    df$NR[i] <- length(cr_strs)
  }

  ## ----- Step 5: optional TC enrichment via NCBI cited-by ----------------
  if (isTRUE(include_TC)) {
    cat("\nEnriching TC via NCBI cited-by...\n")
    pb <- utils::txtProgressBar(min = 0, max = length(pmids), style = 3)
    last_time <- NULL
    for (i in seq_along(pmids)) {
      p <- pmids[i]
      if (is.na(p) || !nzchar(p)) {
        utils::setTxtProgressBar(pb, i)
        next
      }
      last_time <- api_throttle(api_key, last_time)
      cites <- tryCatch(
        pmCitedBy(p, api_key = api_key),
        error = function(e) list(cited_by = character(0), count = 0)
      )
      df$TC[i] <- cites$count
      utils::setTxtProgressBar(pb, i)
    }
    close(pb)
  }

  cat("\nCitation enrichment completed.\n")
  df
}

#' Build a PubMed search query programmatically
#'
#' It helps to build a valid PubMed search query using the Entrez query language,
#' combining multiple search terms with Boolean operators.
#'
#' @param terms is a character or character vector. Search terms to look for in title and abstract fields.
#' @param fields is a character or character vector. PubMed search tags to apply.
#' Default is \code{c("Title/Abstract")}. Common fields include: "Title/Abstract", "Title", "Author",
#' "MeSH Terms", "Affiliation", "Journal".
#' @param language is a character or NULL. Language filter (e.g. "english", "french"). Default is \code{NULL} (no filter).
#' @param pub_type is a character or NULL. Publication type filter (e.g. "Journal Article", "Review", "Clinical Trial").
#' Default is \code{NULL} (no filter).
#' @param date_range is a character vector of length 2 or NULL. Date range in format \code{c("YYYY", "YYYY")} or
#' \code{c("YYYY/MM/DD", "YYYY/MM/DD")}. Default is \code{NULL} (no filter).
#' @param mesh_terms is a character or character vector or NULL. MeSH (Medical Subject Headings) terms.
#' Default is \code{NULL} (no filter).
#' @param author is a character or character vector or NULL. Author names. Default is \code{NULL}.
#' @param journal is a character or character vector or NULL. Journal names or abbreviations. Default is \code{NULL}.
#' @param operator is a character. Boolean operator to combine multiple \code{terms}. One of "AND", "OR".
#' Default is \code{"AND"}.
#'
#' @return a character string containing the formatted PubMed query.
#'
#' @details
#' The function constructs a query string compatible with NCBI's Entrez search system.
#' Multiple terms within the same parameter are combined with the specified \code{operator},
#' while different parameters (terms, language, pub_type, etc.) are combined with AND.
#'
#' For more information about PubMed search tags, visit:
#' \href{https://pubmed.ncbi.nlm.nih.gov/help/#search-tags}{https://pubmed.ncbi.nlm.nih.gov/help/#search-tags}
#'
#' @examples
#'
#' # Simple query
#' q <- pmQueryBuild(terms = "bibliometrics", language = "english",
#'                   pub_type = "Journal Article", date_range = c("2000", "2023"))
#'
#' # Multiple terms
#' q <- pmQueryBuild(terms = c("machine learning", "deep learning"),
#'                   operator = "OR", language = "english")
#'
#' # MeSH terms query
#' q <- pmQueryBuild(mesh_terms = "COVID-19", pub_type = "Review",
#'                   date_range = c("2020", "2024"))
#'
#' # Author search
#' q <- pmQueryBuild(terms = "bibliometrics", author = "Aria M")
#'
#' @seealso \code{\link{pmQueryTotalCount}}
#' @seealso \code{\link{pmApiRequest}}
#'
#' @export
pmQueryBuild <- function(terms = NULL,
                         fields = "Title/Abstract",
                         language = NULL,
                         pub_type = NULL,
                         date_range = NULL,
                         mesh_terms = NULL,
                         author = NULL,
                         journal = NULL,
                         operator = "AND") {

  operator <- match.arg(toupper(operator), choices = c("AND", "OR"))
  parts <- c()

  ## Search terms with field tags
  if (!is.null(terms) && length(terms) > 0) {
    term_parts <- vapply(terms, function(t) {
      field_queries <- vapply(fields, function(f) {
        sprintf("%s[%s]", t, f)
      }, character(1))
      if (length(field_queries) > 1) {
        paste0("(", paste(field_queries, collapse = " OR "), ")")
      } else {
        field_queries
      }
    }, character(1), USE.NAMES = FALSE)

    if (length(term_parts) > 1) {
      parts <- c(parts, paste0("(", paste(term_parts, collapse = paste0(" ", operator, " ")), ")"))
    } else {
      parts <- c(parts, term_parts)
    }
  }

  ## MeSH terms
  if (!is.null(mesh_terms) && length(mesh_terms) > 0) {
    mesh_parts <- vapply(mesh_terms, function(m) {
      sprintf("%s[MeSH Terms]", m)
    }, character(1), USE.NAMES = FALSE)
    if (length(mesh_parts) > 1) {
      parts <- c(parts, paste0("(", paste(mesh_parts, collapse = " OR "), ")"))
    } else {
      parts <- c(parts, mesh_parts)
    }
  }

  ## Author
  if (!is.null(author) && length(author) > 0) {
    au_parts <- vapply(author, function(a) {
      sprintf("%s[Author]", a)
    }, character(1), USE.NAMES = FALSE)
    if (length(au_parts) > 1) {
      parts <- c(parts, paste0("(", paste(au_parts, collapse = " AND "), ")"))
    } else {
      parts <- c(parts, au_parts)
    }
  }

  ## Journal
  if (!is.null(journal) && length(journal) > 0) {
    jo_parts <- vapply(journal, function(j) {
      sprintf("%s[Journal]", j)
    }, character(1), USE.NAMES = FALSE)
    if (length(jo_parts) > 1) {
      parts <- c(parts, paste0("(", paste(jo_parts, collapse = " OR "), ")"))
    } else {
      parts <- c(parts, jo_parts)
    }
  }

  ## Language
  if (!is.null(language) && nzchar(language)) {
    parts <- c(parts, sprintf("%s[LA]", language))
  }

  ## Publication type
  if (!is.null(pub_type) && nzchar(pub_type)) {
    parts <- c(parts, sprintf("%s[PT]", pub_type))
  }

  ## Date range
  if (!is.null(date_range) && length(date_range) == 2) {
    parts <- c(parts, sprintf("%s:%s[DP]", date_range[1], date_range[2]))
  }

  if (length(parts) == 0) {
    stop("At least one search parameter must be provided.", call. = FALSE)
  }

  query <- paste(parts, collapse = " AND ")
  return(query)
}

test_that("pmQueryBuild creates simple query", {
  q <- pmQueryBuild(terms = "bibliometrics", language = "english")
  expect_true(grepl("bibliometrics\\[Title/Abstract\\]", q))
  expect_true(grepl("english\\[LA\\]", q))
  expect_true(grepl("AND", q))
})

test_that("pmQueryBuild handles multiple terms with OR", {
  q <- pmQueryBuild(terms = c("machine learning", "deep learning"), operator = "OR")
  expect_true(grepl("machine learning", q))
  expect_true(grepl("deep learning", q))
  expect_true(grepl("OR", q))
})

test_that("pmQueryBuild handles date range", {
  q <- pmQueryBuild(terms = "cancer", date_range = c("2020", "2024"))
  expect_true(grepl("2020:2024\\[DP\\]", q))
})

test_that("pmQueryBuild handles MeSH terms", {
  q <- pmQueryBuild(mesh_terms = "COVID-19")
  expect_true(grepl("COVID-19\\[MeSH Terms\\]", q))
})

test_that("pmQueryBuild handles author", {
  q <- pmQueryBuild(terms = "bibliometrics", author = "Aria M")
  expect_true(grepl("Aria M\\[Author\\]", q))
})

test_that("pmQueryBuild handles journal filter", {
  q <- pmQueryBuild(terms = "cancer", journal = "Nature")
  expect_true(grepl("Nature\\[Journal\\]", q))
})

test_that("pmQueryBuild handles publication type", {
  q <- pmQueryBuild(terms = "cancer", pub_type = "Review")
  expect_true(grepl("Review\\[PT\\]", q))
})

test_that("pmQueryBuild combines all parameters", {
  q <- pmQueryBuild(
    terms = "bibliometrics",
    language = "english",
    pub_type = "Journal Article",
    date_range = c("2000", "2023"),
    author = "Aria M"
  )
  expect_true(grepl("bibliometrics\\[Title/Abstract\\]", q))
  expect_true(grepl("english\\[LA\\]", q))
  expect_true(grepl("Journal Article\\[PT\\]", q))
  expect_true(grepl("2000:2023\\[DP\\]", q))
  expect_true(grepl("Aria M\\[Author\\]", q))
})

test_that("pmQueryBuild errors with no parameters", {
  expect_error(pmQueryBuild(), "At least one search parameter")
})

test_that("pmQueryBuild handles multiple fields", {
  q <- pmQueryBuild(terms = "cancer", fields = c("Title", "Abstract"))
  expect_true(grepl("Title", q))
  expect_true(grepl("Abstract", q))
})

---
title: "pubmedR"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{pubmedR: A brief example}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---
## An R-package to gather bibliographic data from PubMed.

&nbsp;

    The goal of pubmedR is to gather metadata about publications, grants and
    clinical trials from the PubMed database using NCBI REST APIs.

<!-- badges: start -->
<!-- badges: end -->


&nbsp;

*https://github.com/massimoaria/pubmedR*

*Latest version: `r packageVersion("pubmedR")`, `r Sys.Date()`*

&nbsp;

**by Massimo Aria**

Full Professor in Social Statistics

PhD in Computational Statistics

Laboratory and Research Group STAD Statistics, Technology, Data Analysis

Department of Economics and Statistics

University of Naples Federico II

email aria@unina.it

https://www.massimoaria.com

&nbsp;

## Installation

You can install the development version of pubmedR from [GitHub](https://github.com) with:

    install.packages("devtools")
    devtools::install_github("massimoaria/pubmedR")


You can install the released version of pubmedR from [CRAN](https://CRAN.R-project.org) with:

    install.packages("pubmedR")

&nbsp;

## Load the package


    library(pubmedR)

&nbsp;

## NCBI API key

By default, access to the NCBI API is free and does not strictly require an API key.
Without a key, NCBI limits users to 3 requests per second; registered users with an
API key are allowed up to 10 requests per second.

To obtain a key, register for a ***"my ncbi account"*** (*https://account.ncbi.nlm.nih.gov/*)
and generate one from the ***"account settings page"*** (*https://account.ncbi.nlm.nih.gov/settings/*).

You can pass the key explicitly via the `api_key` argument of any function, or -
preferably - set it once as an environment variable. pubmedR will automatically
pick it up from `PUBMED_API_KEY` or `ENTREZ_KEY`:

    # option 1: pass it explicitly
    api_key <- "your API key"

    # option 2: set it once per session (or in ~/.Renviron)
    Sys.setenv(PUBMED_API_KEY = "your API key")

    # no key
    api_key <- NULL

&nbsp;

# A brief example

Imagine we want to download a metadata collection of journal articles that
(1) use bibliometric approaches, (2) were published in the last two decades,
and (3) are written in English.

Since version 0.1.0, pubmedR offers **two equivalent workflows**:

- a **step-by-step workflow** (`pmQueryBuild` → `pmQueryTotalCount` →
  `pmApiRequest` → `pmApi2df`), which gives you full control over each stage;
- a **one-step workflow** via `pmCollect()`, a convenience wrapper that chains
  the whole pipeline in a single call and optionally enriches the result with
  citation data.

&nbsp;

## One-step workflow: `pmCollect()`

For most use cases, `pmCollect()` is the fastest way to go from a query to a
bibliometrix-ready data frame. It builds the query, checks the total count,
downloads the records, converts the XML into a data frame, and (optionally)
adds citation counts and cited references.

    library(pubmedR)

    M <- pmCollect(
      terms      = "bibliometric*",
      fields     = "Title/Abstract",
      language   = "english",
      pub_type   = "Journal Article",
      date_range = c("2000", "2020"),
      limit      = 2000,
      api_key    = NULL
    )

    # Query: (bibliometric*[Title/Abstract]) AND english[LA] AND
    #        Journal Article[PT] AND 2000:2020[DP]
    #
    # Total records found: 2921
    # Records to download: 2000

You can also pass a raw PubMed query string directly:

    M <- pmCollect(
      query   = "bibliometric*[Title/Abstract] AND english[LA] AND 2023:2024[DP]",
      limit   = 200
    )

Set `enrich = TRUE` to add citation counts (`TC`) and cited references (`CR`)
via `pmEnrichCitations()`. Note that enrichment adds two API calls per record,
so it is best used on smaller collections:

    M <- pmCollect(
      terms      = "bibliometric*",
      date_range = c("2023", "2024"),
      limit      = 50,
      enrich     = TRUE
    )

&nbsp;

## Step-by-step workflow

If you prefer fine-grained control - for example to inspect the translated
query before downloading - you can run the pipeline one stage at a time.

### Step 1: Build the query

Instead of writing the Entrez query string by hand, you can use
`pmQueryBuild()` to compose it programmatically from parameters:

    query <- pmQueryBuild(
      terms      = "bibliometric*",
      fields     = "Title/Abstract",
      language   = "english",
      pub_type   = "Journal Article",
      date_range = c("2000", "2020")
    )

    query
    # [1] "(bibliometric*[Title/Abstract]) AND english[LA] AND
    #      Journal Article[PT] AND 2000:2020[DP]"

Of course, you can still write the query manually if you prefer:

    query <- "bibliometric*[Title/Abstract] AND english[LA] AND Journal Article[PT] AND 2000:2020[DP]"

&nbsp;

### Step 2: Check the effectiveness of the query

Use `pmQueryTotalCount()` to see how many records PubMed would return, along
with the automatically translated query:

    res <- pmQueryTotalCount(query = query, api_key = api_key)

    res$total_count
    # [1] 2921

    res$query_translation
    # [1] "(bibliometric[Title/Abstract] OR bibliometrica[Title/Abstract] OR
    #      ... OR bibliometricstrade[Title/Abstract]) AND english[LA] AND
    #      Journal Article[PT] AND 2000[PDAT] : 2020[PDAT]"

&nbsp;

### Step 3: Download the collection of document metadata

You can now download the whole collection (or a subset, by lowering `limit`):

    D <- pmApiRequest(query = query, limit = res$total_count, api_key = api_key)

    # Documents  200  of  2921
    # Documents  400  of  2921
    # ...
    # Documents  2921  of  2921

`pmApiRequest()` returns a list with the following elements:

- **`data`** - the XML-structured list containing the bibliographic metadata
  collection downloaded from PubMed.
- **`query`** - the original query submitted by the user.
- **`query_translation`** - the query as translated and executed by NCBI's
  Automatic Terms Translation system.
- **`records_downloaded`** - number of records actually downloaded.
- **`total_count`** - total number of records matching the query.

&nbsp;

### Step 4: Convert the XML object into a data frame

Finally, transform the XML-structured object `D` into a data frame where rows
are documents and columns are field tags compatible with the
**bibliometrix R package**
(https://CRAN.R-project.org/package=bibliometrix, https://bibliometrix.org/).

    M <- pmApi2df(D, format = "bibliometrix")

    str(M)
    # 'data.frame':   2918 obs. of  27 variables:
    #  $ AU    : chr  ...
    #  $ AF    : chr  ...
    #  $ TI    : chr  ...
    #  $ SO    : chr  ...
    #  $ LA    : chr  ...
    #  $ DT    : chr  ...
    #  $ DE    : chr  ...
    #  $ AB    : chr  ...
    #  $ C1    : chr  ...
    #  $ TC    : num  ...
    #  $ PY    : num  ...
    #  $ DI    : chr  ...
    #  $ PMID  : chr  ...
    #  ...

Setting `format = "raw"` returns the data frame with all fields in their
native PubMed form instead of the bibliometrix-style field tags.

&nbsp;

## Fetching records by PMID

If you already know which articles you want, you can bypass the query step and
download records directly by their PubMed identifiers with `pmFetchById()`:

    pmids <- c("34813985", "34813456", "34812345")
    D <- pmFetchById(pmids = pmids, api_key = api_key)
    M <- pmApi2df(D)

The returned object follows the same structure as `pmApiRequest()`, so it can
be fed into `pmApi2df()` exactly the same way.

&nbsp;

## Citation enrichment

pubmedR exposes three helpers to retrieve citation information via NCBI's
E-Link service (based on PubMed Central):

- `pmCitedBy(pmid)` - returns the PMIDs of articles citing the given article;
- `pmReferences(pmid)` - returns the PMIDs of articles referenced by the given article;
- `pmEnrichCitations(df)` - adds a `TC` (times cited) column and a `CR` (cited
  references) column to a pubmedR data frame.

Example:

    cites <- pmCitedBy(pmid = "25824007")
    cites$count
    cites$cited_by

    refs <- pmReferences(pmid = "25824007")
    refs$count
    refs$references

    # Add citation counts and references to an existing data frame
    M_enriched <- pmEnrichCitations(M, api_key = api_key)

Note: citation data in PubMed comes from PubMed Central and is less
comprehensive than commercial databases such as Web of Science or Scopus.

&nbsp;

## An overview of the collection using bibliometrix

Once you have a data frame `M`, you can use **bibliometrix** for descriptive
and network analyses
(https://CRAN.R-project.org/package=bibliometrix, https://bibliometrix.org/).

    install.packages("bibliometrix")
    library(bibliometrix)

    results <- biblioAnalysis(M)
    summary(results)

    # Main Information about data
    #
    #  Documents                             2918
    #  Sources (Journals, Books, etc.)       1275
    #  Keywords Plus (ID)                    2245
    #  Author's Keywords (DE)                4212
    #  Period                                2000 - 2020
    #  ...

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pmCitedBy.R
\name{pmCitedBy}
\alias{pmCitedBy}
\title{Find articles that cite a given PubMed article}
\usage{
pmCitedBy(pmid, api_key = NULL)
}
\arguments{
\item{pmid}{is a character or numeric. A single PubMed identifier (PMID).}

\item{api_key}{is a character. It contains a valid API key for the NCBI E-utilities.
Default is \code{api_key=NULL}. The API key can also be set via the environment variable
\code{PUBMED_API_KEY} or \code{ENTREZ_KEY}.}
}
\value{
a list containing:
\tabular{lll}{
pmid \tab \tab The queried PMID.\cr
cited_by \tab \tab A character vector of PMIDs that cite the queried article.\cr
count \tab \tab The number of citing articles found.}
}
\description{
It retrieves the PMIDs of articles that cite a given PubMed article,
using the NCBI E-Link service (PubMed Cited by).
}
\details{
This function uses the NCBI E-Link endpoint with linkname "pubmed_pubmed_citedin"
to find articles in PubMed that cite the given article.

Note: Citation data in PubMed is based on PubMed Central (PMC) and may not be as
comprehensive as commercial citation databases (e.g. Web of Science, Scopus).
}
\examples{

\donttest{
# Find articles that cite PMID 25824007
cites <- pmCitedBy(pmid = "25824007")
cites$count
cites$cited_by
}

}
\seealso{
\code{\link{pmReferences}}

\code{\link{pmFetchById}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pmQueryTotalCount.R
\name{pmQueryTotalCount}
\alias{pmQueryTotalCount}
\title{Count the number of documents returned by a query}
\usage{
pmQueryTotalCount(query, api_key = NULL)
}
\arguments{
\item{query}{is a character. It contains a search query formulated using the Entrez query language.}

\item{api_key}{is a character. It contains a valid API key for the NCBI E-utilities. Default is \code{api_key=NULL}.
The use of NCBI PubMed APIs is entirely free, and doesn't necessarily require an API key.
The API key can also be set via the environment variable \code{PUBMED_API_KEY} or \code{ENTREZ_KEY}.}
}
\value{
a list. It contains three objects:
\tabular{lll}{
total_count \tab  \tab The total number of records returned by the query\cr
query_translation \tab       \tab The query translation by the NCBI Automatic Terms Translation system \cr
web_history \tab      \tab The web history object. The NCBI provides search history features, which is useful for dealing with large lists of IDs or repeated searches.}

To obtain a free access to NCBI API, please visit: \href{https://pmc.ncbi.nlm.nih.gov/tools/developers/}{https://pmc.ncbi.nlm.nih.gov/tools/developers/}
}
\description{
It counts the number of documents that a query returns from the NCBI PubMed database.
}
\examples{

\donttest{
query <- "bibliometric*[Title/Abstract] AND english[LA]
           AND Journal Article[PT] AND 2000:2020[DP]"
D <- pmQueryTotalCount(query = query, api_key = NULL)
}

}
\seealso{
\code{\link{pmApiRequest}}

\code{\link{pmApi2df}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pmCitedBy.R
\name{pmReferences}
\alias{pmReferences}
\title{Find references cited by a given PubMed article}
\usage{
pmReferences(pmid, api_key = NULL)
}
\arguments{
\item{pmid}{is a character or numeric. A single PubMed identifier (PMID).}

\item{api_key}{is a character. It contains a valid API key for the NCBI E-utilities.
Default is \code{api_key=NULL}. The API key can also be set via the environment variable
\code{PUBMED_API_KEY} or \code{ENTREZ_KEY}.}
}
\value{
a list containing:
\tabular{lll}{
pmid \tab \tab The queried PMID.\cr
references \tab \tab A character vector of PMIDs referenced by the queried article.\cr
count \tab \tab The number of references found.}
}
\description{
It retrieves the PMIDs of articles that are cited by (referenced in) a given
PubMed article, using the NCBI E-Link service.
}
\details{
This function uses the NCBI E-Link endpoint with linkname "pubmed_pubmed_refs"
to find articles in PubMed that are referenced by the given article.

Note: Reference data is extracted from PubMed Central (PMC) full-text articles and
is only available when the full text is deposited in PMC. Not all PubMed articles
have reference data available.
}
\examples{

\donttest{
# Find references of PMID 25824007
refs <- pmReferences(pmid = "25824007")
refs$count
refs$references
}

}
\seealso{
\code{\link{pmCitedBy}}

\code{\link{pmFetchById}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pmCollect.R
\name{pmCollect}
\alias{pmCollect}
\title{Collect and process PubMed bibliographic data in one step}
\usage{
pmCollect(
  query = NULL,
  terms = NULL,
  fields = "Title/Abstract",
  language = NULL,
  pub_type = NULL,
  date_range = NULL,
  mesh_terms = NULL,
  limit = 2000,
  enrich = FALSE,
  format = "bibliometrix",
  api_key = NULL,
  batch_size = 200,
  verbose = TRUE
)
}
\arguments{
\item{query}{is a character. A PubMed search query in Entrez syntax.
Alternatively, if \code{terms} is provided, the query is built automatically
using \code{\link{pmQueryBuild}} and this argument is ignored.}

\item{terms}{is a character or character vector or NULL. Search terms passed to
\code{\link{pmQueryBuild}}. When provided, a query is built automatically
and the \code{query} argument is ignored. Default is \code{NULL}.}

\item{fields}{is a character or character vector. PubMed search tags used
when building the query from \code{terms}. Default is \code{"Title/Abstract"}.}

\item{language}{is a character or NULL. Language filter for query building.
Default is \code{NULL}.}

\item{pub_type}{is a character or NULL. Publication type filter for query building.
Default is \code{NULL}.}

\item{date_range}{is a character vector of length 2 or NULL. Date range
in format \code{c("YYYY", "YYYY")}. Default is \code{NULL}.}

\item{mesh_terms}{is a character or character vector or NULL. MeSH terms
for query building. Default is \code{NULL}.}

\item{limit}{is numeric. Maximum number of records to download.
Default is \code{2000}.}

\item{enrich}{is logical. If \code{TRUE}, citation counts (TC) and cited
references (CR) are added via \code{\link{pmEnrichCitations}}.
Default is \code{FALSE} because enrichment makes 2 API calls per article
and can be slow for large collections.}

\item{format}{is a character. Output format passed to \code{\link{pmApi2df}}.
Either \code{"bibliometrix"} (default) or \code{"raw"}.}

\item{api_key}{is a character or NULL. NCBI API key. Can also be set via
the environment variable \code{PUBMED_API_KEY} or \code{ENTREZ_KEY}.
Default is \code{NULL}.}

\item{batch_size}{is numeric. Records per API request. Default is 200.}

\item{verbose}{is logical. If \code{TRUE} (default), prints progress messages.}
}
\value{
a data frame containing bibliographic records, compatible with the
\code{bibliometrix} package when \code{format = "bibliometrix"}.
}
\description{
A convenience wrapper that executes the full pubmedR workflow:
query building, record count check, metadata download, conversion to
data frame, and (optionally) citation enrichment via NCBI E-Link.
}
\details{
This function chains together the core pubmedR functions in the recommended order:
\enumerate{
  \item \strong{Query}: If \code{terms} is provided, builds the query with
        \code{\link{pmQueryBuild}}; otherwise uses the \code{query} string directly.
  \item \strong{Count}: Checks the total number of matching records with
        \code{\link{pmQueryTotalCount}}.
  \item \strong{Download}: Fetches metadata with \code{\link{pmApiRequest}}.
  \item \strong{Convert}: Transforms XML to a data frame with \code{\link{pmApi2df}}.
  \item \strong{Enrich} (optional): Adds citation data with
        \code{\link{pmEnrichCitations}}.
}
}
\examples{

\donttest{
# Using a raw query string
M <- pmCollect(
  query = "bibliometric*[Title/Abstract] AND english[LA] AND 2020:2024[DP]",
  limit = 50
)

# Using the query builder parameters
M <- pmCollect(
  terms = "bibliometric*",
  language = "english",
  pub_type = "Journal Article",
  date_range = c("2020", "2024"),
  limit = 50
)

# With citation enrichment (slower, requires extra API calls)
M <- pmCollect(
  terms = "bibliometric*",
  date_range = c("2023", "2024"),
  limit = 10,
  enrich = TRUE
)
}

}
\seealso{
\code{\link{pmQueryBuild}}, \code{\link{pmQueryTotalCount}},
\code{\link{pmApiRequest}}, \code{\link{pmApi2df}}, \code{\link{pmEnrichCitations}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pmQueryBuild.R
\name{pmQueryBuild}
\alias{pmQueryBuild}
\title{Build a PubMed search query programmatically}
\usage{
pmQueryBuild(
  terms = NULL,
  fields = "Title/Abstract",
  language = NULL,
  pub_type = NULL,
  date_range = NULL,
  mesh_terms = NULL,
  author = NULL,
  journal = NULL,
  operator = "AND"
)
}
\arguments{
\item{terms}{is a character or character vector. Search terms to look for in title and abstract fields.}

\item{fields}{is a character or character vector. PubMed search tags to apply.
Default is \code{c("Title/Abstract")}. Common fields include: "Title/Abstract", "Title", "Author",
"MeSH Terms", "Affiliation", "Journal".}

\item{language}{is a character or NULL. Language filter (e.g. "english", "french"). Default is \code{NULL} (no filter).}

\item{pub_type}{is a character or NULL. Publication type filter (e.g. "Journal Article", "Review", "Clinical Trial").
Default is \code{NULL} (no filter).}

\item{date_range}{is a character vector of length 2 or NULL. Date range in format \code{c("YYYY", "YYYY")} or
\code{c("YYYY/MM/DD", "YYYY/MM/DD")}. Default is \code{NULL} (no filter).}

\item{mesh_terms}{is a character or character vector or NULL. MeSH (Medical Subject Headings) terms.
Default is \code{NULL} (no filter).}

\item{author}{is a character or character vector or NULL. Author names. Default is \code{NULL}.}

\item{journal}{is a character or character vector or NULL. Journal names or abbreviations. Default is \code{NULL}.}

\item{operator}{is a character. Boolean operator to combine multiple \code{terms}. One of "AND", "OR".
Default is \code{"AND"}.}
}
\value{
a character string containing the formatted PubMed query.
}
\description{
It helps to build a valid PubMed search query using the Entrez query language,
combining multiple search terms with Boolean operators.
}
\details{
The function constructs a query string compatible with NCBI's Entrez search system.
Multiple terms within the same parameter are combined with the specified \code{operator},
while different parameters (terms, language, pub_type, etc.) are combined with AND.

For more information about PubMed search tags, visit:
\href{https://pubmed.ncbi.nlm.nih.gov/help/#search-tags}{https://pubmed.ncbi.nlm.nih.gov/help/#search-tags}
}
\examples{

# Simple query
q <- pmQueryBuild(terms = "bibliometrics", language = "english",
                  pub_type = "Journal Article", date_range = c("2000", "2023"))

# Multiple terms
q <- pmQueryBuild(terms = c("machine learning", "deep learning"),
                  operator = "OR", language = "english")

# MeSH terms query
q <- pmQueryBuild(mesh_terms = "COVID-19", pub_type = "Review",
                  date_range = c("2020", "2024"))

# Author search
q <- pmQueryBuild(terms = "bibliometrics", author = "Aria M")

}
\seealso{
\code{\link{pmQueryTotalCount}}

\code{\link{pmApiRequest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pmExtractReferences.R
\name{pmExtractReferences}
\alias{pmExtractReferences}
\title{Extract references from PubMed XML records}
\usage{
pmExtractReferences(P)
}
\arguments{
\item{P}{A list following the PubMed XML structure produced by
\code{pmApiRequest()} or \code{pmFetchById()}. Must contain a \code{$data}
element with one entry per record.}
}
\value{
A data.frame with columns
\tabular{lll}{
source_pmid \tab \tab The PMID of the article that cites the reference.\cr
citation    \tab \tab The free-text \code{<Citation>} string from PubMed.\cr
pmid        \tab \tab The PMID of the cited reference (if available).\cr
doi         \tab \tab The DOI of the cited reference (if available).}

Returns an empty data.frame (with the same schema) if no references are
found.
}
\description{
Walks a result returned by \code{\link{pmApiRequest}} or
\code{\link{pmFetchById}} and pulls the \code{<ReferenceList>} block out of
every record. Returns one row per cited \code{<Reference>}, carrying the
source PMID, the free-text citation, and (when present) the cited PMID and
DOI parsed from \code{<ArticleIdList>}.
}
\details{
Reference data in PubMed XML is populated when the publisher submits a
\code{<ReferenceList>} block to NLM (which is now common, but not universal).
This function does not call any web API; it merely parses what is already
present in the XML. Use \code{\link{pmEnrichCitations}} to also resolve cited
PMIDs into structured WoS-style citation strings.
}
\examples{
\donttest{
D <- pmFetchById("37289732")
refs <- pmExtractReferences(D)
head(refs)
}

}
\seealso{
\code{\link{pmEnrichCitations}}, \code{\link{pmFetchById}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pmApiRequest.R
\name{pmApiRequest}
\alias{pmApiRequest}
\title{Gather bibliographic content from PubMed database using NCBI entrez APIs}
\usage{
pmApiRequest(query, limit, api_key = NULL, batch_size = 200)
}
\arguments{
\item{query}{is a character. It contains a search query formulated using the Entrez query language.}

\item{limit}{is numeric. It indicates the max number of records to download.}

\item{api_key}{is a character. It contains a valid API key for the NCBI E-utilities.
Default is \code{api_key=NULL}. The API key can also be set via the environment variable
\code{PUBMED_API_KEY} or \code{ENTREZ_KEY}.}

\item{batch_size}{is numeric. The number of records to download per API request. Default is 200.}
}
\value{
a list D composed by 5 objects:
\tabular{lll}{
data \tab \tab It is the xml-structured list containing the bibliographic metadata collection downloaded from the PubMed database.\cr
query \tab \tab It a character object containing the original query formulated by the user.\cr
query_translation \tab \tab It a character object containing the query, translated by the NCBI Automatic Terms Translation system and submitted to the PubMed database.\cr
records_downloaded \tab \tab It is an integer object indicating the total number of records downloaded and stored in "data".\cr
total_count \tab \tab It is an integer object indicating the total number of records matching the query (stored in the "query_translation" object").}

To obtain a free access to NCBI API, please visit: \href{https://pmc.ncbi.nlm.nih.gov/tools/developers/}{https://pmc.ncbi.nlm.nih.gov/tools/developers/}

To obtain more information about how to write a NCBI search query, please visit: \href{https://pubmed.ncbi.nlm.nih.gov/help/#search-tags}{https://pubmed.ncbi.nlm.nih.gov/help/#search-tags}
}
\description{
It gathers metadata about publications from the NCBI PubMed database.
The use of NCBI PubMed APIs is entirely free, and doesn't necessarily require an API key.
The function \code{pmApiRequest} queries NCBI PubMed using an entrez query formulated through
the Entrez query language or the helper function \code{\link{pmQueryBuild}}.
}
\details{
Official API documentation is \href{https://www.ncbi.nlm.nih.gov/books/NBK25500/}{https://www.ncbi.nlm.nih.gov/books/NBK25500/}.
}
\examples{

\donttest{
query <- "bibliometric*[Title/Abstract] AND english[LA]
          AND Journal Article[PT] AND 2000:2020[DP]"
 D <- pmApiRequest(query = query, limit = 100, api_key = NULL)
}

}
\seealso{
\code{\link{pmQueryTotalCount}}

\code{\link{pmApi2df}}

\code{\link{pmQueryBuild}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pmCitedBy.R
\name{pmEnrichCitations}
\alias{pmEnrichCitations}
\title{Enrich a PubMed dataframe with citation data}
\usage{
pmEnrichCitations(
  df,
  P = NULL,
  api_key = NULL,
  resolve_pmids = TRUE,
  only_multiple = FALSE,
  include_TC = TRUE,
  batch_size = 200
)
}
\arguments{
\item{df}{is a dataframe. A bibliometric dataframe produced by
\code{\link{pmApi2df}}.}

\item{P}{is the optional list returned by \code{\link{pmApiRequest}} or
\code{\link{pmFetchById}} that produced \code{df}. Reuse it to avoid an
extra round-trip when references are already available. If \code{NULL},
the function calls \code{pmFetchById} on \code{df$PMID} to retrieve the
underlying XML.}

\item{api_key}{is a character. It contains a valid API key for the NCBI
E-utilities. Default is \code{api_key=NULL}. The API key can also be set
via the environment variable \code{PUBMED_API_KEY} or \code{ENTREZ_KEY}.}

\item{resolve_pmids}{logical. When \code{TRUE} (default) the function fetches
metadata for every cited PMID present in the references and assembles
structured WoS-style citation strings ("AUTHOR YYYY, JOURNAL, V##, P##,
DOI ..."). When \code{FALSE} the free-text \code{<Citation>} block from
the XML is used as-is.}

\item{only_multiple}{logical. When \code{TRUE} only references cited by more
than one source article are resolved to metadata (faster and cheaper).
References not resolved keep their free-text citation. Defaults to
\code{FALSE}.}

\item{include_TC}{logical. When \code{TRUE} (default) also call
\code{\link{pmCitedBy}} for each source article and write the citation
count to \code{df$TC}. Disable to skip this step.}

\item{batch_size}{integer. Number of records per API call when fetching
metadata. Defaults to 200 (NCBI's hard cap for efetch).}
}
\value{
The input dataframe with updated \code{CR} (cited references),
  \code{NR} (number of references), and \code{TC} (times cited, if
  \code{include_TC = TRUE}) fields.
}
\description{
Adds cited references (CR field), reference counts (NR field), and
optionally citation counts (TC field) to a dataframe created by
\code{\link{pmApi2df}}.
}
\details{
Cited references are extracted from the article's PubMed XML
(\code{<ReferenceList>}). This is more reliable than the previous E-Link
\code{pubmed_pubmed_refs} approach, which only worked for articles
deposited in PMC. References whose XML carries an \code{ArticleId}
\code{IdType="pubmed"} are resolved to bibliographic metadata in
batched \code{efetch} requests so that \code{CR} matches the WoS
convention used by bibliometrix; references with only free-text citations
are kept verbatim (uppercased).
}
\examples{
\donttest{
query <- "bibliometric*[Title/Abstract] AND english[LA]
         AND Journal Article[PT] AND 2000:2020[DP]"
D <- pmApiRequest(query = query, limit = 10, api_key = NULL)
M <- pmApi2df(D)
M <- pmEnrichCitations(M, P = D)        # avoid the extra fetch
}

}
\seealso{
\code{\link{pmExtractReferences}}, \code{\link{pmCitedBy}},
  \code{\link{pmFetchById}}, \code{\link{pmApi2df}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pmApi2df.R
\name{pmApi2df}
\alias{pmApi2df}
\title{Convert xml PubMed bibliographic data into a dataframe}
\usage{
pmApi2df(P, format = "bibliometrix")
}
\arguments{
\item{P}{is a list following the xml PubMed structure, downloaded using the function \code{pmApiRequest}.}

\item{format}{is a character. If \code{format = "bibliometrix"} data will be converted in the bibliometrix compatible data format.
If \code{format = "raw"} data will save in a data frame without any other data editing procedure.}
}
\value{
a dataframe containing bibliographic records.

To obtain a free access to NCBI API, please visit: \href{https://pmc.ncbi.nlm.nih.gov/tools/developers/}{https://pmc.ncbi.nlm.nih.gov/tools/developers/}

To obtain more information about how to write a NCBI search query, please visit: \href{https://pubmed.ncbi.nlm.nih.gov/help/#search-tags}{https://pubmed.ncbi.nlm.nih.gov/help/#search-tags}
}
\description{
It converts PubMed data, downloaded using Entrez API, into a dataframe
}
\examples{

# Example: Querying a collection of publications

\donttest{
query <- "bibliometric*[Title/Abstract] AND english[LA]
         AND Journal Article[PT] AND 2000:2020[DP]"
D <- pmApiRequest(query = query, limit = 100, api_key = NULL)
M <- pmApi2df(D)
}

}
\seealso{
\code{\link{pmApiRequest}}

\code{\link{pmQueryTotalCount}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pmFetchById.R
\name{pmFetchById}
\alias{pmFetchById}
\title{Fetch PubMed records by PMID}
\usage{
pmFetchById(pmids, api_key = NULL, batch_size = 200)
}
\arguments{
\item{pmids}{is a character or numeric vector. A vector of PubMed identifiers (PMIDs).}

\item{api_key}{is a character. It contains a valid API key for the NCBI E-utilities.
Default is \code{api_key=NULL}. The API key can also be set via the environment variable
\code{PUBMED_API_KEY} or \code{ENTREZ_KEY}.}

\item{batch_size}{is numeric. The number of records to download per API request. Default is 200.}
}
\value{
a list following the same structure as \code{\link{pmApiRequest}} output, containing:
\tabular{lll}{
data \tab \tab The xml-structured list containing the bibliographic metadata.\cr
query \tab \tab A character string describing the PMID-based query.\cr
query_translation \tab \tab Same as query for PMID-based searches.\cr
records_downloaded \tab \tab The total number of records downloaded.\cr
total_count \tab \tab The total number of PMIDs requested.}
}
\description{
It downloads metadata for a set of PubMed articles identified by their PMID (PubMed Identifier).
This is useful for retrieving specific known articles, updating existing datasets,
or downloading records identified through other sources.
}
\details{
The function uses the NCBI E-utilities \code{efetch} endpoint to retrieve records directly
by their PMIDs, without requiring a search query. Records are downloaded in batches
to respect API rate limits.

The output is compatible with \code{\link{pmApi2df}} for conversion to a dataframe.
}
\examples{

\donttest{
# Download specific articles by PMID
pmids <- c("34813985", "34813456", "34812345")
D <- pmFetchById(pmids = pmids)
M <- pmApi2df(D)
}

}
\seealso{
\code{\link{pmApiRequest}}

\code{\link{pmApi2df}}
}
